@@ -5,8 +5,8 @@
 #' For the generalised linear mixed model 
 #' 
 #' \deqn{Y \sim F(\mu,\sigma)}
-#' \deqn{\mu = h^-1(X\beta + Z\gamma)}
-#' \deqn{\gamma \sim MVN(0,D)}
+#' \deqn{\mu = h^-1(X\beta + Zu)}
+#' \deqn{u \sim MVN(0,D)}
 #' 
 #' where h is the link function. A Model in comprised of a \link[glmmrBase]{MeanFunction} object, which defines the family F, 
 #' link function h, and fixed effects design matrix X, and a \link[glmmrBase]{Covariance} object, which defines Z and D. The class provides
@@ -15,17 +15,25 @@
 #' This class provides methods for generating the matrices described above and data simulation, and serves as a base for extended functionality 
 #' in related packages.
 #' 
-#' The class by default calculates the covariance matrix of the observations as:
+#' Many calculations use the covariance matrix of the observations, such as the information matrix, which is used in power calculations and 
+#' other functions. For non-Gaussian models, the class uses the approximation proposed by Breslow and Clayton (1993) based on the 
+#' marginal quasilikelihood:
 #' 
 #' \deqn{\Sigma = W^{-1} + ZDZ^T}
 #' 
-#' where _W_ is a diagonal matrix with the WLS iterated weights for each observation equal
-#' to, for individual _i_ \eqn{\phi a_i v(\mu_i)[h'(\mu_i)]^2} (see Table 2.1 in McCullagh 
-#' and Nelder (1989) <ISBN:9780412317606>). For very large designs, this can be disabled as
-#' the memory requirements can be prohibitive. 
+#' where _W_ is a diagonal matrix with the GLM iterated weights for each observation equal
+#' to, for individual _i_ \eqn{\left( \frac{(\partial h^{-1}(\eta_i))}{\partial \eta_i}\right) ^2 Var(y|u)} 
+#' (see Table 2.1 in McCullagh and Nelder (1989) <ISBN:9780412317606>). For very large designs, this can be disabled as
+#' the memory requirements can be prohibitive (use option `skip_sigma`). 
 #' 
 #' See \href{https://github.com/samuel-watson/glmmrBase/blob/master/README.md}{glmmrBase} for a 
 #' detailed guide on model specification.
+#' @references 
+#' Breslow, N. E., Clayton, D. G. (1993). Approximate Inference in Generalized Linear Mixed Models. 
+#' Journal of the American Statistical Association<, 88(421), 9–25. <doi:10.1080/01621459.1993.10594284>
+#' 
+#' Zeger, S. L., Liang, K.-Y., Albert, P. S. (1988). Models for Longitudinal Data: A Generalized Estimating Equation Approach. 
+#' Biometrics, 44(4), 1049.<doi:10.2307/2531734>
 #' @importFrom Matrix Matrix
 #' @export 
 Model <- R6::R6Class("Model",
@@ -34,6 +42,8 @@ Model <- R6::R6Class("Model",
                        covariance = NULL,
                        #' @field mean_function A \link[glmmrBase]{MeanFunction} object, defining the mean function for the model, including the data and covariate design matrix X.
                        mean_function = NULL,
+                       #' @field family One of the family function used in R's glm functions. See \link[stats]{family} for details
+                       family = NULL,
                        #' @field exp_condition A vector indicting the unique experimental conditions for each observation, see Details.
                        exp_condition = NULL,
                        #' @field Sigma The overall covariance matrix for the observations. Calculated and updated automatically as \eqn{W^{-1} + ZDZ^T} where W is an n x n 
@@ -42,6 +52,30 @@ Model <- R6::R6Class("Model",
                        #' @field var_par Scale parameter required for some distributions (Gaussian, Gamma, Beta).
                        var_par = NULL,
                        #' @description 
+                       #' Sets the model to use or not use "attenuation" when calculating the first-order approximation to 
+                       #' the covariance matrix. 
+                       #' @details 
+                       #' **Attenuation**
+                       #' For calculations such as the information matrix, the first-order approximation to the covariance matrix 
+                       #' proposed by Breslow and Clayton (1993), described above, is used. The approximation is based on the 
+                       #' marginal quasilikelihood. Zegers, Liang, and Albert (1988) suggest that a better approximation to the 
+                       #' marginal mean is achieved by "attenuating" the linear predictor. Setting `use` equal to TRUE uses this 
+                       #' adjustment for calculations using the covariance matrix for non-linear models.
+                       #' @param use Logical indicating whether to use "attenuation".
+                       #' @return None. Used for effects.
+                       use_attenuation = function(use){
+                         curr_state <- private$attenuate_parameters
+                         if(use){
+                           private$attenuate_parameters <- TRUE
+                         } else {
+                           private$attenuate_parameters <- FALSE
+                         }
+                         if(private$attenuate_parameters != curr_state){
+                           private$genW()
+                           private$genS()
+                         }
+                       },
+                       #' @description 
                        #' Return predicted values based on the currently stored parameter values in `mean_function`
                        #' @param type One of either "`link`" for values on the scale of the link function, or "`response`" 
                        #' for values on the scale of the response
@@ -49,7 +83,7 @@ Model <- R6::R6Class("Model",
                        fitted = function(type="link"){
                          Xb <- Matrix::drop(self$mean_function$X %*% self$mean_function$parameters)
                          if(type=="response"){
-                           Xb <- self$mean_function$family$linkinv(Xb)
+                           Xb <- self$family$linkinv(Xb)
                          }
                          return(Xb)
                        },
@@ -155,6 +189,12 @@ Model <- R6::R6Class("Model",
                            }
                          }
                          
+                         if(is.null(family)){
+                           stop("No family specified.")
+                         } else {
+                           self$family <- family
+                         }
+                         
                          if(is(mean,"R6")){
                            if(is(mean,"MeanFunction")){
                              self$mean_function <- mean
@@ -165,13 +205,6 @@ Model <- R6::R6Class("Model",
                                  self$mean_function$data <- data
                                }
                              }
-                             if(is.null(mean$family)){
-                               if(is.null(family)){
-                                 stop("No family specified in MeanFunction object or call to function.")
-                               } else {
-                                 self$mean_function$family <- family
-                               }
-                             }
                              
                            } else {
                              stop("mean should be MeanFunction class or list of appropriate arguments")
@@ -179,7 +212,6 @@ Model <- R6::R6Class("Model",
                          } else if(is(mean,"list")){
                            if(is.null(mean$formula))stop("A formula must be specified for the mean function.")
                            if(is.null(mean$data) & is.null(data))stop("No data specified in mean list or call to function.")
-                           if(is.null(mean$family) & is.null(family))stop("No family specified in mean list or call to function.")
                            self$mean_function <- MeanFunction$new(
                              formula = mean$formula
                            )
@@ -190,11 +222,6 @@ Model <- R6::R6Class("Model",
                            } else {
                              self$mean_function$data <- mean$data
                            }
-                           if(is.null(mean$family)){
-                             self$mean_function$family <- family 
-                           } else {
-                             self$mean_function$family <- mean$family
-                           }
                          }
                          
                          if(!is.null(var_par)){
@@ -214,11 +241,17 @@ Model <- R6::R6Class("Model",
                        #' Calls the respective print methods of the linked covariance and mean function objects.
                        #' @param ... ignored
                        print = function(){
-                         cat("\n----------------------------------------\n")
-                         print(self$mean_function)
-                         cat("\n----------------------------------------\n")
-                         print(self$covariance)
-                         cat("\n----------------------------------------\n")
+                         cat("\U2BC8 GLMM Model")
+                         cat("\n   \U2BA1 Family :",self$family[[1]])
+                         cat("\n   \U2BA1 Link :",self$family[[2]])
+                         cat("\n   \U2BA1 Linear predictor")
+                         cat("\n   \U2223     \U2BA1 Formula: ~",as.character(self$mean_function$formula)[2])
+                         cat("\n   \U2223     \U2BA1 Parameters: ",self$mean_function$parameters)
+                         cat("\n   \U2BA1 Covariance")
+                         cat("\n   \U2223     \U2BA1 Formula: ~",as.character(self$covariance$formula)[2])
+                         cat("\n   \U2223     \U2BA1 Parameters: ",self$covariance$parameters)
+                         cat("\n   \U2223     \U2BA1 N random effects: ",ncol(self$covariance$Z))
+                         cat("\n   \U2BA1 N:",self$n())
                        },
                        #' @description 
                        #' Returns the number of observations in the model
@@ -302,9 +335,11 @@ Model <- R6::R6Class("Model",
                        #'
                        #'Generates a single vector of outcome data based upon the 
                        #'specified GLMM design
-                       #'@param type Either 'y' to return just the outcome data, or 'data'
-                       #' to return a data frame with the simulated outcome data alongside the model data 
-                       #' @return Either a vector or a data frame
+                       #'@param type Either 'y' to return just the outcome data, 'data'
+                       #' to return a data frame with the simulated outcome data alongside the model data,
+                       #' or 'all', which will return a list with simulated outcomes y, matrices X and Z, 
+                       #' parameters beta, and the values of the simulated random effects.
+                       #' @return Either a vector, a data frame, or a list
                        #' @examples
                        #' df <- nelder(~(cl(10)*t(5)) > ind(10))
                        #' df$int <- 0
@@ -326,7 +361,7 @@ Model <- R6::R6Class("Model",
                                                              gamma = self$covariance$parameters)))
                          mu <- c(drop(as.matrix(self$mean_function$X)%*%self$mean_function$parameters)) + c(as.matrix(self$covariance$Z)%*%re)
                          
-                         f <- self$mean_function$family
+                         f <- self$family
                          if(f[1]=="poisson"){
                            if(f[2]=="log"){
                              y <- rpois(self$n(),exp(mu))
@@ -364,7 +399,7 @@ Model <- R6::R6Class("Model",
                          }
                          
                          
-                         if(f[1]=="gamma"){
+                         if(f[1]=="Gamma"){
                            if(f[2]=="inverse"){
                              if(is.null(self$var_par))stop("For gamma(link='inverse') provide var_par")
                              y <- rgamma(self$n(),shape = self$var_par,rate = self$var_par*mu)
@@ -388,6 +423,8 @@ Model <- R6::R6Class("Model",
                          }
                          
                          if(type=="data.frame"|type=="data")y <- cbind(y,self$mean_function$data)
+                         if(type=="all")y <- list(y = y, X = self$mean_function$X, beta = self$mean_function$parameters,
+                                                  Z = self$covariance$Z, u = re)
                          return(y)
                          
                        },
@@ -523,50 +560,50 @@ Model <- R6::R6Class("Model",
                          exp(x)/(1+exp(x))
                        },
                        generate = function(){
-                         # add check for var par with gaussian family
-                         
-                         private$genW(family = self$mean_function$family,
-                                      Xb = self$mean_function$.__enclos_env__$private$Xb,
-                                      var_par = self$var_par)
-                         private$genS(D = self$covariance$D,
-                                      Z = self$covariance$Z,
-                                      W = private$W)
+                         private$genW()
+                         private$genS()
                        },
-                       genW = function(family,
-                                       Xb,
-                                       var_par=NULL){
+                       genW = function(){
                          # assume random effects value is at zero
-                         if(!family[[1]]%in%c("poisson","binomial","gaussian","Gamma","beta"))stop("family must be one of Poisson, Binomial, Gaussian, Gamma, Beta")
+                         if(!self$family[[1]]%in%c("poisson","binomial","gaussian","Gamma","beta"))stop("family must be one of Poisson, Binomial, Gaussian, Gamma, Beta")
+                         xb <- c(self$mean_function$.__enclos_env__$private$Xb)
+                         if(private$attenuate_parameters){
+                           xb <- attenuate_xb(xb = xb,
+                                              Z = as.matrix(self$covariance$Z),
+                                              D = as.matrix(self$covariance$D),
+                                              link = self$family[[2]])
+                         }
+                         wdiag <- gen_dhdmu(xb = xb,
+                                            family=self$family[[1]],
+                                            link = self$family[[2]])
                          
-                         wdiag <- gen_dhdmu(c(Xb),
-                                            family=family[[1]],
-                                            link = family[[2]])
-                         
-                         if(family[[1]] == "gaussian"){
-                           wdiag <- var_par * var_par * wdiag
-                         } else if(family[[1]] == "Gamma"){
-                           wdiag <- wdiag/var_par
-                         } else if(family[[1]] == "beta"){
-                           wdiag <- wdiag*(1+var_par)
+                         if(self$family[[1]] == "gaussian"){
+                           wdiag <- self$var_par * self$var_par * wdiag
+                         } else if(self$family[[1]] == "Gamma"){
+                           wdiag <- wdiag/self$var_par
+                         } else if(self$family[[1]] == "beta"){
+                           wdiag <- wdiag*(1+self$var_par)
                          }
                          
                          W <- diag(drop(wdiag))
                          private$W <- Matrix::Matrix(W)
                        },
-                       genS = function(D,Z,W,update=TRUE){
-                         if(is(D,"numeric")){
-                           S <- W + D * Matrix::tcrossprod(Z)
-                         } else {
-                           S <- W + Z %*% Matrix::tcrossprod(D,Z)
-                         }
+                       genS = function(update=TRUE){
+                         S = gen_sigma_approx(xb=matrix(self$mean_function$X%*%self$mean_function$parameters,ncol=1),
+                                              Z = as.matrix(self$covariance$Z),
+                                              D = as.matrix(self$covariance$D),
+                                              family=self$family[[1]],
+                                              link = self$family[[2]],
+                                              var_par = self$var_par,
+                                              attenuate = private$attenuate_parameters)
                          if(update){
                            self$Sigma <- Matrix::Matrix(S)
                            private$hash <- private$hash_do()
                          } else {
                            return(S)
                          }
-                         
                        },
+                       attenuate_parameters = TRUE,
                        hash = NULL,
                        hash_do = function(){
                          digest::digest(c(self$covariance$.__enclos_env__$private$hash,
