@@ -1,6 +1,6 @@
 #' Generate a staircase/diagonal trial design
 #' 
-#' Generate a staircase/diagonal cluster randomised trial design in glmmr
+#' Generate a staircase/diagonal cluster randomised trial design model
 #' 
 #' The staircase/diagonal cluster randomised trial design has J sequences of clusters
 #' observed over J time periods, each sequence has `nper` clusters. 
@@ -12,11 +12,10 @@
 #' @param J Integer indicating the number of sequences such that there are J time periods
 #' @param M Integer. The number of individual observations per cluster-period, assumed equal across all clusters
 #' @param beta Vector of beta parameters to initialise the design, defaults to all zeros.
-#' @param icc Intraclass correlation coefficient. User may specify
-#' more than one value, see details.
-#' @param cac Cluster autocorrelation coefficient, optional and user may specify more than one value, see details
-#' @param iac Individual autocorrelation coefficient, optional and user may specify more than one value, see details
-#' @param var Assumed overall variance of the model, used to calculate the other covariance, see details
+#' @param icc Intraclass correlation coefficient. 
+#' @param cac Cluster autocorrelation coefficient, optional
+#' @param iac Individual autocorrelation coefficient, optional
+#' @param var Assumed overall variance of the model, used to calculate the other covariance parameters, see details
 #' @param family a \link[stats]{family} object 
 #' @details 
 #' The assumed generalised linear mixed model for the staircase/diagonal cluster trial is, for 
@@ -44,9 +43,6 @@
 #' might include the variance at the mean values of the parameters or a reasonable choice based
 #' on the variance of the respective distribution.
 #' 
-#' If the user specifies more than one value for icc, cac, or iac, then a ModelSpace is returned
-#' with Models with every combination of parameters. This can be used in particular to generate
-#' a design space for optimal design analyses.
 #' @examples 
 #' #generate a simple design with only cluster random effects and 6 clusters with 10
 #' #individuals in each cluster-period
@@ -54,10 +50,7 @@
 #' # same design but with a cohort of individuals
 #' des <- staircase_crt(6,10,icc=0.05, iac = 0.1)
 #' # same design, but with two clusters per sequence and specifying the initial parameters
-#' des <- staircase_crt(6,10,beta = c(rnorm(7,0,0.1),-0.1),icc=0.05, iac = 0.1)
-#' # specifying multiple values of the variance parameters will return a design space 
-#' # with all designs with all the combinations of the variance parameter
-#' des <- staircase_crt(6,10,icc=c(0.01,0.05), cac = c(0.5,0.7,0.9), iac = 0.1)
+#' des <- staircase_crt(6,10,beta = c(rnorm(5,0,0.1),-0.1),icc=0.05, iac = 0.1)
 #' @return A Model object with MeanFunction and Covariance objects, or
 #' a ModelSpace holding several such Model objects.
 #' @seealso \link[glmmrBase]{Model}
@@ -66,7 +59,7 @@
 #' @export
 staircase_crt <-  function(J,
                            M,
-                           beta=c(rep(0,J+1),0),
+                           beta=c(rep(0,J-1),0),
                            icc,
                            cac = NULL,
                            iac = NULL,
@@ -74,23 +67,20 @@ staircase_crt <-  function(J,
                            family = stats::gaussian()){
   if(missing(icc))stop("icc must be set as a minimum")
   
-  ndesigns <- length(icc) * ifelse(!is.null(cac[1]),length(cac),1) *
-    ifelse(!is.null(iac[1]),length(iac),1)
-  
-  if(!is.null(cac[1]) && !is.na(cac[1])){
-    wp_var <- icc[1]*var*(1-cac[1])
-    bp_var <- icc[1]*var[1]*cac[1]
+  if(!is.null(cac) && !is.null(cac)){
+    wp_var <- icc*var*(1-cac)
+    bp_var <- icc*var*cac
   } else {
-    bp_var <- icc[1]*var
+    bp_var <- icc*var
   }
-  if(!is.null(iac[1]) && !is.na(iac[1])){
-    ind_var <- var*(1-icc[1])*iac[1]
-    sigma <- var*(1-icc[1])*(1-iac[1])
+  if(!is.null(iac) && !is.null(iac)){
+    ind_var <- var*(1-icc)*iac
+    sigma <- var*(1-icc)*(1-iac)
   } else {
-    sigma <- var*(1-icc[1])
+    sigma <- var*(1-icc)
   }
   t <- J+1
-  if(!is.null(iac[1]) && !is.na(iac[1])){
+  if(!is.null(iac) && !is.null(iac)){
     df <- nelder(formula(paste0("~ (J(",J,") > ind(",M,")) * t(",t,")")))
   } else {
     df <- nelder(formula(paste0("~ (J(",J,") * t(",t,")) > ind(",M,")")))
@@ -133,79 +123,17 @@ staircase_crt <-  function(J,
   
   d1 <- Model$new(
     covariance = list(
-      data=df,
       formula = f1,
       parameters = pars
     ),
     mean = list(
       formula = "~ factor(t) + int - 1",
-      data = df,
-      family = family,
       parameters = beta
       
     ),
+    data = df,
+    family = family,
     var_par = sigma
   )
-  
-  # if(ndesigns>1&requireNamespace(glmmrOptim)){
-  #   ds1 <- ModelSpace$new(d1)
-  #   if(is.null(cac))cac <- NA
-  #   if(is.null(iac))iac <- NA
-  #   dsvalues <- expand.grid(icc=icc,cac=cac,iac=iac)
-  #   
-  #   for(i in 1:(ndesigns-1)){
-  # 
-  #     if(!is.null(dsvalues$cac[i+1]) && !is.na(dsvalues$cac[i+1])){
-  #       wp_var <- dsvalues$icc[i+1]*var*(1-dsvalues$cac[i+1])
-  #       bp_var <- dsvalues$icc[i+1]*var*dsvalues$cac[i+1]
-  #     } else {
-  #       bp_var <- dsvalues$icc[i+1]*var
-  #     }
-  #     if(!is.null(dsvalues$iac[i+1]) && !is.na(dsvalues$iac[i+1])){
-  #       ind_var <- var*(1-dsvalues$icc[i+1])*dsvalues$iac[i+1]
-  #       sigma <- var*(1-dsvalues$icc[i+1])*(1-dsvalues$iac[i+1])
-  #     } else {
-  #       sigma <- var*(1-dsvalues$icc[i+1])
-  #     }
-  #     
-  #     if(is.null(dsvalues$cac[i+1]) || is.na(dsvalues$cac[i+1])){
-  #       if(is.null(dsvalues$iac[i+1]) || is.na(dsvalues$iac[i+1])){
-  #         f1 <- "~(1|gr(J))"
-  #         pars <- c(sqrt(bp_var))
-  #       } else {
-  #         f1 <- "~(1|gr(J)) + (1|gr(ind))"
-  #         pars <- c(sqrt(bp_var),sqrt(ind_var))
-  #       }
-  #     } else {
-  #       if(is.null(dsvalues$iac[i+1]) || is.na(dsvalues$iac[i+1])){
-  #         f1 <- "~ (1|gr(J)) + (1|gr(J*t))"
-  #         pars <- c(sqrt(bp_var),sqrt(wp_var))
-  #       } else {
-  #         f1 <- "~ (1|gr(J)) + (1|gr(J*t)) + (1|gr(ind))"
-  #         pars <- c(sqrt(bp_var),sqrt(wp_var),sqrt(ind_var))
-  #       }
-  #     }
-  #     
-  #     
-  #     ds1$add(
-  #       Model$new(
-  #         covariance = Covariance$new(
-  #           data=df,
-  #           formula = f1,
-  #           parameters = pars
-  #         ),
-  #         mean.function = d1$mean_function$clone(),
-  #         var_par = 1
-  #       )
-  #     )
-  #     
-  #     # $parameters <- pars
-  #     # ds1$.__enclos_env__$private$designs[[i+1]]$covariance$formula <- f1
-  #     
-  #   }
-  #   return(ds1)
-  # } else {
-  #   return(d1)
-  # }
   return(invisible(d1))
 }
